"""
Command to create seeds for ResumeManager app models.
"""

# Third-Party Imports.
from django.core.management.base import BaseCommand

# Internal Imports.
from . import resume_manager_core__loadfixtures
from workspace_core.settings.reusable_settings import ConsoleColors


# Initialize fixture class.
app_fixtures = resume_manager_core__loadfixtures.Command()


class Command(BaseCommand):
    help = 'Seed database models with randomized data for ResumeManager app.'

    def add_arguments(self, parser):
        """
        Parser for command.
        """
        # Optional arguments.
        parser.add_argument(
            'model_count',
            type=int,
            nargs='?',
            default=100,
            help='Number of randomized models to create. Defaults to 100. Cannot exceed 10,000.',
        )

    def handle(self, *args, **kwargs):
        """
        The logic of the command.
        """
        # Limit range of model_count variable.
        model_count = kwargs['model_count']
        if model_count < 1:
            model_count = 100
        elif model_count > 10000:
            model_count = 100

        print('{0}RESUME_MANAGER{1}: {2}Seed{3} command has been called.'.format(
            ConsoleColors.purple,
            ConsoleColors.reset,
            ConsoleColors.blue,
            ConsoleColors.reset,
        ))

        # Run fixture imports.
        self.create_resume_themes(model_count)
        self.create_resumes(model_count)
        self.create_section_css(model_count)
        self.create_resume_sections(model_count)
        self.create_contact_info(model_count)
        self.create_education_instances(model_count)
        self.create_experience_instances(model_count)
        self.create_skill_categories(model_count)
        self.create_skill_instances(model_count)
        self.create_reference_instances(model_count)

        print('{0}RESUME_MANAGER{1}: {2}Seeding{3} complete.\n'.format(
            ConsoleColors.purple,
            ConsoleColors.reset,
            ConsoleColors.blue,
            ConsoleColors.reset,
        ))

    def create_resume_themes(self, model_count):
        """
        Create seeds for ResumeTheme models.
        """
        # Load preset fixtures.
        app_fixtures.populate_resume_themes()

        # Seed logic goes here.

    def create_resumes(self, model_count):
        """
        Create seeds for Resume models.
        """
        # Load preset fixtures.
        app_fixtures.populate_resumes()

        # Seed logic goes here.

    def create_section_css(self, model_count):
        """
        Create seeds for SectionCss models.
        """
        # Load preset fixtures.
        app_fixtures.populate_section_css()

        # Seed logic goes here.

    def create_resume_sections(self, model_count):
        """
        Create seeds for ResumeSection models.
        """
        # Load preset fixtures.
        app_fixtures.populate_resume_sections()

        # Seed logic goes here.

    def create_contact_info(self, model_count):
        """
        Create seeds for ContactInfo models.
        """
        # Load preset fixtures.
        app_fixtures.populate_contact_info()

        # Seed logic goes here.

    def create_education_instances(self, model_count):
        """
        Create seeds for EducationInstance models.
        """
        # Load preset fixtures.
        app_fixtures.populate_education_instances()

        # Seed logic goes here.

    def create_experience_instances(self, model_count):
        """
        Create seeds for ExperienceInstance models.
        """
        # Load preset fixtures.
        app_fixtures.populate_experience_instances()

        # Seed logic goes here.

    def create_skill_categories(self, model_count):
        """
        Create seeds for SkillCategory models.
        """
        # Load preset fixtures.
        app_fixtures.populate_skill_categories()

        # Seed logic goes here.

    def create_skill_instances(self, model_count):
        """
        Create seeds for SkillInstance models.
        """
        # Load preset fixtures.
        app_fixtures.populate_skill_instances()

        # Seed logic goes here.

    def create_reference_instances(self, model_count):
        """
        Create seeds for ReferenceInstance models.
        """
        # Load preset fixtures.
        app_fixtures.populate_reference_instances()

        # Seed logic goes here.
