"""
Models for ResumeManager app.

NOTE: While many of these models could be generalized by removing the
    "resume" FK relation, they are intentionally left as-is, so that a user
    can completely tailor each resume 100% separately from others.

    In other words, the redundancy ensures that prior resume values have
    no forced implications on future resumes. Values can be copied if desired,
    but can also be wholy unique from individual resume to resume.
"""

# Third-Party Imports.
import phonenumbers
from django.db import models
from django.conf import settings
from django.core.exceptions import ValidationError
from django.utils import timezone
from phonenumber_field.modelfields import PhoneNumberField

# Internal Imports.
from core.models import (
    EmailField,
    SmallIntegerRangeField,
    WebsiteField,
    WorkspaceModel,
)


class Resume(WorkspaceModel):
    """A representation of a resume."""

    # Relationship Keys.
    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)
    theme = models.ForeignKey('ResumeTheme', on_delete=models.CASCADE)

    # Model fields.
    name = models.CharField(max_length=settings.MAX_LENGTH)
    profession = models.CharField(max_length=settings.MAX_LENGTH, blank=True, null=True)
    last_activity = models.DateField(default=timezone.now)

    # Field helper text.
    name.help_text = 'Resume name.'
    last_activity.help_text = 'Last set date of resume updating. Defaults to creation date. Manually set after.'

    class Meta:
        verbose_name = 'Resume'
        verbose_name_plural = 'Resumes'

    def __str__(self):
        return '{0} - {1}'.format(self.name, self.last_activity)


class ResumeTheme(WorkspaceModel):
    """A unique resume theme."""

    # Model fields.
    name = models.CharField(max_length=settings.MAX_LENGTH)
    css_classes = models.CharField(max_length=settings.MAX_LENGTH)
    has_header_image = models.BooleanField(default=False)
    has_footer_image = models.BooleanField(default=False)

    # Field helper text.
    name.help_text = 'Name of theme.'
    has_header_image.help_text = 'Indicates if theme includes header image.'
    has_footer_image.help_text = 'Indicates if theme includes footer image.'

    class Meta:
        verbose_name = 'Resume Theme'
        verbose_name_plural = 'Resume Themes'

    def __str__(self):
        return '{0}'.format(self.name)


class ResumeSection(WorkspaceModel):
    """Formatting of a section/subsection within a resume."""

    # Preset field choices.
    DISPLAY_CATEGORY__NONE = 0
    DISPLAY_CATEGORY__NAME = 1
    DISPLAY_CATEGORY__PROFESSION = 2
    DISPLAY_CATEGORY__CONTACT = 3
    DISPLAY_CATEGORY__EDUCATION = 4
    DISPLAY_CATEGORY__EXPERIENCE = 5
    DISPLAY_CATEGORY__SKILLS = 6
    DISPLAY_CATEGORY__REFERENCES = 7
    DISPLAY_CATEGORY_CHOICES = (
        (DISPLAY_CATEGORY__NONE, 'None'),
        (DISPLAY_CATEGORY__CONTACT, 'Name'),
        (DISPLAY_CATEGORY__CONTACT, 'Profession'),
        (DISPLAY_CATEGORY__CONTACT, 'Contact'),
        (DISPLAY_CATEGORY__EDUCATION, 'Education'),
        (DISPLAY_CATEGORY__EXPERIENCE, 'Experience'),
        (DISPLAY_CATEGORY__SKILLS, 'Skills'),
        (DISPLAY_CATEGORY__REFERENCES, 'References'),
    )

    # Relationship Keys.
    css = models.ManyToManyField('SectionCss', related_name='section')
    resume = models.ForeignKey('Resume', on_delete=models.CASCADE)
    parent_section = models.ForeignKey('self', on_delete=models.CASCADE, blank=True, null=True)

    # Model fields.
    name = models.CharField(max_length=settings.MAX_LENGTH)
    description = models.CharField(max_length=settings.MAX_LENGTH, blank=True, null=True)
    display_category = models.PositiveSmallIntegerField(choices=DISPLAY_CATEGORY_CHOICES, default=0)
    separator_before = models.BooleanField(default=False)
    separator_after = models.BooleanField(default=False)

    # Field helper text.
    css.help_text = 'CSS stylings to apply to section.'
    resume.help_text = 'Resume that section applies to.'
    parent_section.help_text = 'Parent section/subsection to nest within.'
    name.help_text = 'Name to help identify section purpose.'
    description.help_text = 'Brief description line to show at top of resume section, if desired.'
    display_category.help_text = 'Indicates what content (if any) should display in section.'
    separator_before.help_text = 'Indicates if a separator should be placed before section elements.'
    separator_after.help_text = 'indicates if a separator should be placed after section elements.'

    class Meta:
        verbose_name = 'Resume Section'
        verbose_name_plural = 'Resume Sections'

    def __str__(self):
        return '{0}'.format(self.name)


class SectionCss(WorkspaceModel):
    """Visual style formatting of a section/subsection within a resume."""

    # Model fields.
    name = models.CharField(max_length=settings.MAX_LENGTH)
    css_classes = models.CharField(max_length=settings.MAX_LENGTH)

    # Field helper text.
    name.help_text = 'Name of resume section.'
    css_classes.help_text = 'CSS classes to apply for section.'

    class Meta:
        verbose_name = 'Section CSS'
        verbose_name_plural = 'Section CSS'

    def __str__(self):
        return '{0}'.format(self.name)


class ContactInfo(WorkspaceModel):
    """"""
    # Relationship Keys.
    resume = models.ForeignKey('Resume', on_delete=models.CASCADE)

    # Model fields.
    phone_number = PhoneNumberField()
    email = EmailField(max_length=settings.MAX_LENGTH)
    website = WebsiteField(max_length=settings.MAX_LENGTH, blank=True, null=True)
    git = models.CharField(max_length=settings.MAX_LENGTH, blank=True, null=True)
    linkedin = models.CharField(max_length=settings.MAX_LENGTH, blank=True, null=True)
    art_station = models.CharField(max_length=settings.MAX_LENGTH, blank=True, null=True)

    # Field helper text.
    resume.help_text = 'Resume that contact info applies to. Allows user to have different contact info per resume.'
    website.help_text = 'Personal website, or other professional site not provided here.'
    git.help_text = 'Site to showcase personal git projects.'
    linkedin.help_text = 'LinkedIn site profile.'
    art_station.help_text = 'ArtStation site profile.'

    class Meta:
        verbose_name = 'Contact Info'
        verbose_name_plural = 'Contact Info'

    def __str__(self):
        return '{0} {1} - {2}'.format(self.resume.user.first_name, self.resume.user.last_name, self.resume.name)


class EducationInstance(WorkspaceModel):
    """An instance of educational experience."""

    # Relationship Keys.
    resume = models.ForeignKey('Resume', on_delete=models.CASCADE)

    # Model fields.
    name = models.CharField(max_length=settings.MAX_LENGTH)
    degree = models.CharField(max_length=settings.MAX_LENGTH)
    gpa = models.CharField(max_length=settings.MAX_LENGTH, blank=True, null=True)
    date_start = models.DateField()
    date_end = models.DateField(blank=True, null=True)
    expected_graduation_date = models.DateField(blank=True, null=True)
    order = SmallIntegerRangeField(default=0, min_value=0, max_value=100)

    # Field helper text.
    name.help_text = 'Name of institution.'
    degree.help_text = 'Degree pursued/acquired.'
    date_start.help_text = 'Starting date of pursuing education.'
    date_end.help_text = 'Ending date of finishing education, or none if ongoing.'
    order.help_text = 'Ordering in relation to other education instances in the same resume.'

    class Meta:
        verbose_name = 'Education Instance'
        verbose_name_plural = 'Education Instances'
        ordering = ('resume', 'order', 'date_end', 'date_start', 'name', 'degree', 'id')

    def __str__(self):
        return '{0} - {1}'.format(self.name, self.degree)

    def clean(self, *args, **kwargs):
        """
        Custom cleaning implementation. Includes validation, setting fields, etc.
        """
        # Verify date_end (if present) is after date_start.
        if self.date_end and self.date_start > self.date_end:
            raise ValidationError('End date cannot come before Start date.')

        # Verify graduation_date is blank if date_end is populated.
        if self.date_end and self.expected_graduation_date:
            self.expected_graduation_date = None

        # Verify GPA is blank if date_end is not populated.
        if not self.date_end:
            self.gpa = None

        # Verify expected_graduation_date (if present) is after date_start.
        if self.date_end and self.date_start > self.date_end:
            raise ValidationError('Expected graduation date cannot come before start date.')


class ExperienceInstance(WorkspaceModel):
    """An instance of work/job experience."""

    # Relationship Keys.
    resume = models.ForeignKey('Resume', on_delete=models.CASCADE)

    # Model fields.
    name = models.CharField(max_length=settings.MAX_LENGTH)
    position = models.CharField(max_length=settings.MAX_LENGTH)
    description = models.TextField(max_length=500, blank=True, null=True)
    date_start = models.DateField()
    date_end = models.DateField(blank=True, null=True)
    order = SmallIntegerRangeField(default=0, min_value=0, max_value=100)

    # Field helper text.
    name.help_text = 'Name of company/workplace.'
    position.help_text = 'Title of position.'
    description.help_text = 'Description of position.'
    date_start.help_text = 'Initial hiring date.'
    date_end.help_text = 'Termination date, or none if ongoing.'
    order.help_text = 'Ordering in relation to other experience instances in the same resume.'

    class Meta:
        verbose_name = 'Experience Instance'
        verbose_name_plural = 'Experience Instances'
        ordering = ('resume', 'order', 'date_end', 'date_start', 'name', 'position', 'id')

    def __str__(self):
        return '{0} - {1}'.format(self.name, self.position)

    def clean(self, *args, **kwargs):
        """
        Custom cleaning implementation. Includes validation, setting fields, etc.
        """
        if self.date_end and self.date_start > self.date_end:
            raise ValidationError('Start date cannot come after end date.')


class SkillCategory(WorkspaceModel):
    """An instance of applicable skills."""

    # Relationship Keys.
    resume = models.ForeignKey('Resume', on_delete=models.CASCADE)

    # Model fields.
    name = models.CharField(max_length=settings.MAX_LENGTH)
    order = SmallIntegerRangeField(default=0, min_value=0, max_value=100)

    # Field helper text.
    name.help_text = 'Name of skill category.'
    name.help_text = 'Ordering in relation to other skill categories in the same resume.'

    class Meta:
        verbose_name = 'Skill Category'
        verbose_name_plural = 'Skill Categories'
        ordering = ('resume', 'order', 'name', 'id')
        unique_together = (
            ['resume', 'name'],
        )

    def __str__(self):
        return '{0} - {1}'.format(self.resume, self.name)


class SkillInstance(WorkspaceModel):
    """An instance of applicable skills."""

    # Relationship Keys.
    category = models.ForeignKey('SkillCategory', on_delete=models.CASCADE)

    # Model fields.
    name = models.CharField(max_length=settings.MAX_LENGTH)

    class Meta:
        verbose_name = 'Skill Instance'
        verbose_name_plural = 'Skill Instances'
        ordering = ('category', 'name', 'id')
        unique_together = (
            ['category', 'name'],
        )

    def __str__(self):
        return '{0} - {1} - {2}'.format(self.category.resume, self.category, self.name)


class ReferenceInstance(WorkspaceModel):
    """An instance of a contact reference."""

    # Relationship Keys.
    resume = models.ForeignKey('Resume', on_delete=models.CASCADE)

    # Model fields.
    name = models.CharField(max_length=settings.MAX_LENGTH)
    company = models.CharField(max_length=settings.MAX_LENGTH)
    phone_number = PhoneNumberField()
    email = EmailField(max_length=settings.MAX_LENGTH)
    order = SmallIntegerRangeField(default=0, min_value=0, max_value=100)

    # Field helper text.
    name.help_text = 'Reference\'s name.'
    company.help_text = 'Reference\'s employment/job title.'
    phone_number.help_text = 'Reference\'s contact phone.'
    email.help_text = 'Reference\'s contact email.'
    order.help_text = 'Ordering in relation to other reference instances in the same resume.'

    class Meta:
        verbose_name = 'Reference Instance'
        verbose_name_plural = 'Reference Instances'
        ordering = ('resume', 'order', 'name', 'company', 'id')

    def __str__(self):
        return '{0} - {1}'.format(self.resume, self.name)
